************************************************************************
*                                                                      *
*  These routines depend to some extent on the parametrisations for    *
*  the unpolarised parton densities, and require modification when     *
*  these sets are changed.                                             *
*                                                                      *
************************************************************************
      subroutine initf1(y)
************************************************************************
*                                                                      *
*     Construct the initial values for unpolarised parton densities.   *
*                                                                      *
*        This routine is based on CTEQ4L parametrizations with the     *
*        simple valence distributions.                                 *
*                                                                      *
*     Version of Jan 12, 2000.                                         *
*                                                                      *
************************************************************************
$include apeno.h
$include grids.h
$include cteq4m.h
$include fixpar.h
      dimension y(*)
************************************************************************
*     Initialise the parton densities at Q_0^2.                        *
************************************************************************
      call inipars
      do 10 ix=1,nxgrd
         x=x0(ix)
         y(ix+m1u*nxgrd)=unm*(x**(a1u-1))*(1+a3u*(x**a4u))*(1-x)**a2u
         y(ix+m1d*nxgrd)=dnm*(x**(a1d-1))*(1+a3d*(x**a4d))*(1-x)**a2d
         y(ix+m1q0*nxgrd)=2*(2+eta)*a0q*(x**(a1q-1))*(1+a3q*x**a4q)
     1       *(1-x)**a2q
         y(ix+m1q3*nxgrd)=2*a0qp*(x**(a1qp-1))*(1+a3qp*sqrt(x)
     1       +a4qp*x)*(1-x)**a2qp
         y(ix+m1q8*nxgrd)=2*(1-eta)*y(ix+m1q0*nxgrd)/(2+eta)
         y(ix+m1q15*nxgrd)=y(ix+m1q0*nxgrd)
 10      y(ix+m1g*nxgrd)=gnm*(x**(a1g-1))*(1+a3g*x**a4g)*(1-x)**a2g
*
      return
      end
      subroutine inipars
************************************************************************
*                                                                      *
*     Initialise the various fixed parameters of the distributions.    *
*                                                                      *
************************************************************************
$include cteq4m.h
$include fixpar.h
$include integs.h
************************************************************************
*     The unpolarised normalisations: impose three sum rules.          *
************************************************************************
      eta=0.5
      unm=2.0/fn(a1u-1,a2u,a3u,a4u)
      dnm=1.0/fn(a1d-1,a2d,a3d,a4d)
      gnm=(1-unm*fn(a1u,a2u,a3u,a4u)-dnm*fn(a1d,a2d,a3d,a4d)
     1      -2*(2+eta)*a0q*fn(a1q,a2q,a3q,a4q))/fn(a1g,a2g,a3g,a4g)
      write(*,'(a,e14.6)') '# unm=', unm, '# dnm=',
     1   dnm, '# gnm=', gnm, '# eta=', eta
*
      return
      end
      subroutine initg1(y,par,penalty)
************************************************************************
*                                                                      *
*     Construct the initial values for the polarised parton densities. *
*                                                                      *
*     Version of Jan 14, 2000.                                         *
*                                                                      *
************************************************************************
$include apeno.h
$include cteq4m.h
$include fixpar.h
$include grids.h
      parameter (xtreme=1.0E+05)
      logical positive
      dimension y(*)
      double precision par(*)
************************************************************************
*     Impose the Bjorken sum rule.                                     *
*        (note that this defines q3-bar = 2 (ubar - dbar)              *
************************************************************************
      call bjorken(par)
      if (penalty.lt.0.) write(*,'(a,e12.4)') '# dpol0 =', dpol0
      if (positive(par)) then	! positivity not enforced for
         penalty=0.		! the NLO program ...
      else
         penalty=xtreme
      end if
************************************************************************
*     Initialise the parton densities at Q_0^2.                        *
************************************************************************
      do 10 ix=1,nxgrd
         x=x0(ix)
         y(ix+m1u*nxgrd)=par(1)*(x**(par(2)+a1u-1))
     1       *(1+par(3)*(x**a4u))*(1-x)**a2u
         y(ix+m1d*nxgrd)=par(4)*(x**(par(2)+a1d-1))
     1       *(1+par(5)*(x**a4d))*(1-x)**a2d
         y(ix+m1q0*nxgrd)=2*(2+eta)*par(6)*(x**(par(7)-1))
     1       *(1+par(8)*x**a4q)*(1-x)**a2q
         y(ix+m1q3*nxgrd)=dpol0*(x**(par(2)+a1qp-1))
     1       *(1+a3qp*sqrt(x)+par(9)*x)*(1-x)**a2qp
         y(ix+m1q8*nxgrd)=2*(1-eta)*y(ix+m1q0*nxgrd)/(2+eta)
         y(ix+m1q15*nxgrd)=y(ix+m1q0*nxgrd)
         y(ix+m1g*nxgrd)=par(10)*(x**(par(11)-1))*(1+par(12)*x**a4g)
     1       *(1-x)**a2g
 10      continue
*
      return
      end
      subroutine bjorken(par)
************************************************************************
*                                                                      *
*     Use the Bjorken sum rule to determine a0d(polarised).            *
*     The sum rule evaluated to NLO requires no alpha_s.               *
*                                                                      *
*     Version of Feb 6, 2000.                                          *
*                                                                      *
************************************************************************
      parameter (ga=1.2670)
      parameter (xtreme=1.0E+02)
      double precision par(*)
$include cteq4m.h
$include fixpar.h
$include grids.h
$include integs.h
************************************************************************
*     Impose the Bjorken sum rule.                                     *
************************************************************************
      expo1=par(2)+a1u
      expo2=par(2)+a1d
      expo3=par(2)+a1qp
*
      if ((expo1.le.0.).or.(expo2.le.0.).or.(expo3.le.0.)) then
         dpol0=xtreme
         return
      end if
*
      sea = f2(expo3-1.0,a2qp,a3qp,par(9))
      if (abs(sea).le.1.E-3) then
         dpol0=xtreme
         return
      end if
*
      uval=par(1)*fn(expo1-1.0,a2u,par(3),a4u)
      dval=par(4)*fn(expo2-1.0,a2d,par(5),a4d)
      dpol0=(ga-uval+dval)/sea
*
      return
      end
      logical function positive(par)
************************************************************************
*                                                                      *
*     Check whether all densities satisfy positivity bounds.           *
*                                                                      *
*     Version of Jan 12, 2000.                                         *
*                                                                      *
************************************************************************
$include cteq4m.h
$include fixpar.h
      double precision par(*)
      logical bound
*
      a0=par(1)
      a1=a1u+par(2)
      a3=par(3)
      positive=bound(a0,a1,a3,unm,a1u,a3u,a4u)
c     write(*,*) 'u valence', positive
      if (.not.positive) return		! valence u
*
      a0=par(4)
      a1=a1d+par(2)
      a3=par(5)
      positive=bound(a0,a1,a3,dnm,a1d,a3d,a4d)
c     write(*,*) 'd valence', positive
      if (.not.positive) return		! valence d
*
      a0=par(6)
      a1=par(7)
      a3=par(8)
      positive=bound(a0,a1,a3,a0q,a1q,a3q,a4q)
c     write(*,*) 'sea', positive
      if (.not.positive) return		! sea quarks
*
      a0=dpol0/2
      a1=a1qp+par(2)
      a3=par(9)
      positive=bound(a0,a1,a3,a0qp,a1qp,a4qp,1.0)
c     write(*,*) 'triplet sea', positive
      if (.not.positive) return		! triplet sea quarks
*
      a0=par(10)
      a1=par(11)
      a3=par(12)
      positive=bound(a0,a1,a3,gnm,a1g,a3g,a4g) ! gluons
c     write(*,*) 'gluon', positive
*
      return
      end
      logical function bound(a0,a1,a3,b0,b1,b3,d)
************************************************************************
*                                                                      *
*     Check whether the functional form in the arithmetic statement    *
*     function f(t) is bounded by unity.                               *
*                                                                      *
************************************************************************
      f(t)=abs(a0*(t**(a1-b1))*(1+a3*(t**d))/(1+b3*(t**d))/b0)
************************************************************************
*                                                                      *
*     If both parts of the function are increasing then check.         *
*     If the power-law part is decreasing then function unbounded.     *
*                                                                      *
************************************************************************
      if (a1.lt.b1) then
c        write(*,*) 'a1<b1'
         bound=.false.
         return
      else if (a3.gt.b3) then
c        write(*,*) 'a3>b3'
         bound=f(1.0).le.1.0
         return
      else if (a1.eq.b1) then
c        write(*,*) 'a1=b1'
         bound=f(0.0).le.1.0
         return
      end if
************************************************************************
*                                                                      *
*     The power-law part of the function is increasing and the ratio   *
*     of linear parts is decreasing. Check whether f'(t)/f(t) vanishes *
*     for real t. If not, then the function is monotonic. Check if     *
*     it is bounded at t=1.0.                                          *
*                                                                      *
************************************************************************
      c0=a1-b1
      c1=a3*(d+c0)+b3*(c0-d)
      c2=a3*b3*c0
      det=c1*c1-4*c0*c2
      if (det.le.0.0) then
         bound=f(1.0).le.1.0
         return
      end if
************************************************************************
*                                                                      *
*     The extremum of f(x) occurs for real x. If it occurs for x<0     *
*     then check boundedness at x=0; if it occurs for x>1 then check   *
*     boundedness for x=1; otherwise check at the solution x.          *
*                                                                      *
************************************************************************
      if (abs(c2/c1).le.1.E-3) then
         x=-c0/c1
      else
         x=0.5*(-c1+sqrt(det))/c2
      end if
*
      if (x.le.0.) then
         bound=f(1.0).le.1.0
      else
         x=exp(log(x)/d)
         if (x.gt.1.) then
            bound=f(1.0).le.1.0
         else
            bound=f(x).le.1.0
         end if
      end if
*
      if (.not.bound) return
*
      x=0.5*(-c1-sqrt(det))/c2
      if (x.le.0.) then
         bound=f(1.0).le.1.0
      else
         x=exp(log(x)/d)
         if (x.gt.1.) then
            bound=f(1.0).le.1.0
         else
            bound=f(x).le.1.0
         end if
      end if
*
      return
      end
      subroutine thrsh(alsq5,alsq4,alsq3)
************************************************************************
*                                                                      *
*     Compute mass threshold effects.                                  *
*                                                                      *
************************************************************************
      parameter (epsnewt=1.E-5)
$include cteq4m.h
$include grids.h
$include qcd.h
************************************************************************
*                                                                      *
*     Construct flavour dependent values of Lambda at NLO.             *
*                                                                      *
************************************************************************
      if (nflam.eq.5) then
         alsq5=alam*alam
         write(*,'(a,3e12.4)') '# Lambda5=', sqrt(alsq5), bmass
*
         t=log(bmass*bmass/alsq5)
         alf0=(1-(bt15/bt05**2)*log(t)/t)/(bt05*t)
         t0=(bt05*t)/bt04
 10      t4=t0*(bt14+bt04*bt04*t0*(2-alf0*bt04*t0)-3*bt14*log(t0))
     1            /(bt14+bt04*bt04*t0-2*bt14*log(t0))
            if (abs(t0-t4).gt.epsnewt) then
               t0=t4
               go to 10
            end if
         alsq4=bmass*bmass*exp(-t4)
         write(*,'(a,3e12.4)') '# Lambda4=', sqrt(alsq4), cmass
*
         t=log(cmass*cmass/alsq4)
         alf0=(1-(bt14/bt04**2)*log(t)/t)/(bt04*t)
         t0=(bt04*t)/bt03
 11      t3=t0*(bt13+bt03*bt03*t0*(2-alf0*bt03*t0)-3*bt13*log(t0))
     1            /(bt13+bt03*bt03*t0-2*bt13*log(t0))
            if (abs(t0-t3).gt.epsnewt) then
               t0=t3
               go to  11
            end if
         alsq3=cmass*cmass*exp(-t3)
         write(*,'(a,3e12.4)') '# Lambda3=', sqrt(alsq3)
      end if
      q2(0)=q0sq
************************************************************************
*                                                                      *
*     Guard for unphysical evolution across the charm threshold.       *
*                                                                      *
************************************************************************
      q2c=cmass*cmass
      k3=iq2sch(q2c)
      if (abs(q2(k3)-q2c).lt.1.E-4) then
         do 20 iq=nqgrd,k3+1,-1
 20         q2(iq+1)=q2(iq)
         q2(k3+1)=q2c
         nqgrd=nqgrd+1
      else
         do 21 iq=nqgrd,k3+1,-1
 21         q2(iq+2)=q2(iq)
         q2(k3+1)=q2c
         q2(k3+2)=q2c
         k3=k3+1
         nqgrd=nqgrd+2
      end if
************************************************************************
*                                                                      *
*     Guard for unphysical evolution across the bottom threshold.      *
*                                                                      *
************************************************************************
      q2b=bmass*bmass
      k4=iq2sch(q2b)
      if (abs(q2(k4)-q2b).lt.1.E-4) then
         do 30 iq=nqgrd,k4+1,-1
 30         q2(iq+1)=q2(iq)
         q2(k4+1)=q2b
         nqgrd=nqgrd+1
      else
         do 31 iq=nqgrd,k4+1,-1
 31         q2(iq+2)=q2(iq)
         q2(k4+1)=q2b
         q2(k4+2)=q2b
         k4=k4+1
         nqgrd=nqgrd+2
      end if
************************************************************************
*                                                                      *
*     Keep track of the effective number of flavours at each qsq.      *
*                                                                      *
************************************************************************
      do 40 iq=0,k3
 40      neff(iq)=3
*
      do 41 iq=k3+1,k4
 41      neff(iq)=4
*
      do 42 iq=k4+1,nqgrd
 42      neff(iq)=5
*
      return
      end
